import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { Component, TemplateRef, ElementRef, ViewChild, EventEmitter, Directive, Input, Output, ComponentFactoryResolver, ViewContainerRef, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ObserversModule } from '@angular/cdk/observers';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapPopupPlaceholderComponent {
    constructor() {
        this.contentChanged = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get inserted() {
        return this._inserted;
    }
    /**
     * @param {?} inserted
     * @return {?}
     */
    set inserted(inserted) {
        this._inserted = inserted;
        if (inserted instanceof TemplateRef) {
            this._template = inserted;
        }
        else {
            this._template = this.defaultTemplate;
        }
    }
    /**
     * @return {?}
     */
    get template() {
        return this._template;
    }
}
NgxBootstrapPopupPlaceholderComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-bootstrap-popup-placeholder',
                template: `
    <div #insertedContent (cdkObserveContent)="contentChanged.emit($event)">
      <ng-container *ngTemplateOutlet="template"></ng-container>
    </div>
    <ng-template #defaultTemplate>{{inserted}}</ng-template>
  `,
                styles: [':host {display: none;}']
            }] }
];
NgxBootstrapPopupPlaceholderComponent.propDecorators = {
    defaultTemplate: [{ type: ViewChild, args: ['defaultTemplate',] }],
    insertedContent: [{ type: ViewChild, args: ['insertedContent',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class NgxBootstrapPopup {
    /**
     * @param {?} _elementRef
     * @param {?} _cfr
     * @param {?} _vcr
     */
    constructor(_elementRef, _cfr, _vcr) {
        this._elementRef = _elementRef;
        this._cfr = _cfr;
        this._vcr = _vcr;
        this._ngUnsubscribe = new Subject();
        this._showing = false;
        this.clickDismissListener = null;
        this.events = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get titleComponentRef() {
        return this._titleComponentRef;
    }
    /**
     * @return {?}
     */
    get contentComponentRef() {
        return this._contentComponentRef;
    }
    /**
     * @return {?}
     */
    get cfr() {
        return this._cfr;
    }
    /**
     * @return {?}
     */
    get vcr() {
        return this._vcr;
    }
    /**
     * @return {?}
     */
    get el() {
        return this._elementRef.nativeElement;
    }
    /**
     * @return {?}
     */
    get $el() {
        return jQuery(this.el);
    }
    /**
     * @return {?}
     */
    get bsInstance() {
        return this.$el.data('bs.' + this.popupType);
    }
    /**
     * @return {?}
     */
    get showing() {
        return this._showing;
    }
    /**
     * @return {?}
     */
    get jQueryEvents() {
        return ['show', 'inserted', 'shown', 'hide', 'hidden'].map(((/**
         * @param {?} t
         * @return {?}
         */
        t => `${t}.bs.${this.popupType}`))).join(' ');
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.create();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._ngUnsubscribe.next();
        this._ngUnsubscribe.complete();
        this.$el[this.popupType]('dispose');
        this.$el.off(this.jQueryEvents, this.bsEventListener);
        if (this.clickDismissListener) {
            jQuery('body').off('click focusin', this.clickDismissListener);
        }
        if (this.titleComponentRef) {
            this.titleComponentRef.destroy();
        }
        if (this.contentComponentRef) {
            this.contentComponentRef.destroy();
        }
    }
    /**
     * @return {?}
     */
    create() {
        this.bsEventListener = ((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            if (this.dismissOnClickOutside && event.type === 'shown') {
                this.clickDismissListener = ((/**
                 * @param {?} clickEvent
                 * @return {?}
                 */
                (clickEvent) => {
                    if (this.bsInstance.tip.contains(clickEvent.target)) {
                        return;
                    }
                    jQuery('body').off('click focusin', this.clickDismissListener);
                    this.clickDismissListener = null;
                    this.hide();
                }));
                jQuery('body').on('click focusin', this.clickDismissListener);
            }
            if (event.type === 'show') {
                this._showing = true;
            }
            if (event.type === 'hidden') {
                jQuery('body').off('click focusin', this.clickDismissListener);
                this.clickDismissListener = null;
                this._showing = false;
            }
            this.events.emit(event);
        }));
        this.$el.on(this.jQueryEvents, this.bsEventListener);
        /** @type {?} */
        const options = this.getOptions();
        this.$el[this.popupType](options);
        this.updateEnabled();
    }
    /**
     * @return {?}
     */
    getOptions() {
        /** @type {?} */
        const options = Object.assign({}, this.options);
        if (this.title) {
            this._titleComponentRef = this.getPlaceholderComponent();
            this.titleComponentRef.instance.inserted = this.title;
            options.title = this.titleComponentRef.instance.insertedContent.nativeElement;
            options.html = true;
        }
        if (this.content) {
            this._contentComponentRef = this.getPlaceholderComponent();
            this.contentComponentRef.instance.inserted = this.content;
            options.content = this.contentComponentRef.instance.insertedContent.nativeElement;
            options.html = true;
        }
        return options;
    }
    /**
     * @return {?}
     */
    getPlaceholderComponent() {
        /** @type {?} */
        const factory = this.cfr.resolveComponentFactory(NgxBootstrapPopupPlaceholderComponent);
        /** @type {?} */
        const ref = this.vcr.createComponent(factory, 0);
        ref.instance.contentChanged
            .pipe(takeUntil(this._ngUnsubscribe))
            .subscribe((/**
         * @return {?}
         */
        () => {
            if (this.showing) {
                this.update();
            }
        }));
        return ref;
    }
    /**
     * @return {?}
     */
    show() {
        this.$el[this.popupType]('show');
    }
    /**
     * @return {?}
     */
    hide() {
        this.$el[this.popupType]('hide');
    }
    /**
     * @return {?}
     */
    toggle() {
        this.$el[this.popupType]('toggle');
    }
    /**
     * @return {?}
     */
    enable() {
        this.$el[this.popupType]('enable');
    }
    /**
     * @return {?}
     */
    disable() {
        this.$el[this.popupType]('disable');
    }
    /**
     * @return {?}
     */
    toggleEnabled() {
        this.$el[this.popupType]('toggleEnabled');
    }
    /**
     * @return {?}
     */
    update() {
        this.$el[this.popupType]('update');
    }
    /**
     * @return {?}
     */
    updateTitle() {
        if (this.titleComponentRef && this.title) {
            this.titleComponentRef.instance.inserted = this.title;
        }
    }
    /**
     * @return {?}
     */
    updateContent() {
        if (this.contentComponentRef && this.content) {
            this.contentComponentRef.instance.inserted = this.content;
        }
    }
    /**
     * @return {?}
     */
    updateEnabled() {
        if (this.bsInstance) {
            if (this.enabled) {
                this.enable();
            }
            else {
                this.disable();
            }
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapPopoverDirective extends NgxBootstrapPopup {
    /**
     * @param {?} elementRef
     * @param {?} cfr
     * @param {?} vcr
     */
    constructor(elementRef, cfr, vcr) {
        super(elementRef, cfr, vcr);
        this.popoverEnabled = true;
        this.popoverDismissOnClickOutside = true;
        this.popoverEvents = this.events;
    }
    /**
     * @return {?}
     */
    get popupType() {
        return 'popover';
    }
    /**
     * @return {?}
     */
    get content() {
        return this.popoverContent || null;
    }
    /**
     * @return {?}
     */
    get title() {
        return this.popoverTitle || null;
    }
    /**
     * @return {?}
     */
    get dismissOnClickOutside() {
        return this.popoverDismissOnClickOutside;
    }
    /**
     * @return {?}
     */
    get enabled() {
        return this.popoverEnabled;
    }
    /**
     * @return {?}
     */
    get options() {
        return this.popoverOptions || null;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.popoverEnabled !== undefined) {
            this.updateEnabled();
        }
        if (changes.popoverTitle) {
            this.updateTitle();
        }
        if (changes.popoverContent) {
            this.updateContent();
        }
    }
}
NgxBootstrapPopoverDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxBootstrapPopover]'
            },] }
];
/** @nocollapse */
NgxBootstrapPopoverDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ComponentFactoryResolver },
    { type: ViewContainerRef }
];
NgxBootstrapPopoverDirective.propDecorators = {
    popoverTitle: [{ type: Input }],
    popoverContent: [{ type: Input }],
    popoverEnabled: [{ type: Input }],
    popoverDismissOnClickOutside: [{ type: Input }],
    popoverOptions: [{ type: Input }],
    popoverEvents: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapTooltipDirective extends NgxBootstrapPopup {
    /**
     * @param {?} elementRef
     * @param {?} cfr
     * @param {?} vcr
     */
    constructor(elementRef, cfr, vcr) {
        super(elementRef, cfr, vcr);
        this.tooltipEnabled = true;
        this.tooltipDismissOnClickOutside = true;
        this.tooltipEvents = this.events;
    }
    /**
     * @return {?}
     */
    get popupType() {
        return 'tooltip';
    }
    /**
     * @return {?}
     */
    get content() {
        return null;
    }
    /**
     * @return {?}
     */
    get title() {
        return this.tooltipTitle || null;
    }
    /**
     * @return {?}
     */
    get dismissOnClickOutside() {
        return this.tooltipDismissOnClickOutside;
    }
    /**
     * @return {?}
     */
    get enabled() {
        return this.tooltipEnabled;
    }
    /**
     * @return {?}
     */
    get options() {
        return this.tooltipOptions || null;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.tooltipEnabled !== undefined) {
            this.updateEnabled();
        }
        if (changes.tooltipTitle) {
            this.updateTitle();
        }
    }
}
NgxBootstrapTooltipDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxBootstrapTooltip]'
            },] }
];
/** @nocollapse */
NgxBootstrapTooltipDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ComponentFactoryResolver },
    { type: ViewContainerRef }
];
NgxBootstrapTooltipDirective.propDecorators = {
    tooltipTitle: [{ type: Input }],
    tooltipEnabled: [{ type: Input }],
    tooltipDismissOnClickOutside: [{ type: Input }],
    tooltipOptions: [{ type: Input }],
    tooltipEvents: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapPopupModule {
}
NgxBootstrapPopupModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    NgxBootstrapPopupPlaceholderComponent,
                    NgxBootstrapTooltipDirective,
                    NgxBootstrapPopoverDirective,
                ],
                imports: [
                    CommonModule,
                    ObserversModule
                ],
                exports: [
                    NgxBootstrapTooltipDirective,
                    NgxBootstrapPopoverDirective
                ],
                entryComponents: [
                    NgxBootstrapPopupPlaceholderComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxBootstrapPopoverDirective, NgxBootstrapTooltipDirective, NgxBootstrapPopup, NgxBootstrapPopupModule, NgxBootstrapPopupPlaceholderComponent as ɵa };

//# sourceMappingURL=nowzoo-ngx-bootstrap-popup.js.map